"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFulfilledResults = exports.getThrottleResValues = exports.convertISODateToEDIDate = exports.throttle = exports.optimizeOrderCustomAttributes = exports.updateTagsForMultipleOrders = exports.updateTags = exports.markOrderAsPaid = exports.formatTax = exports.getOrder = exports.getOrders = exports.graphqlClient = void 0;
const graphql_request_1 = require("graphql-request");
exports.graphqlClient = new graphql_request_1.GraphQLClient(`https://${process.env.SHOPIFY_PASSWORD}:${process.env.SHOPIFY_USERNAME}/admin/api/2021-07/graphql.json`, {
    headers: {
        "Content-Type": "application/json",
    },
});
async function getOrders({ query, orderParams, limit = 250, }) {
    try {
        const res1 = await exports.graphqlClient.request(`{
              orders(first: ${limit},query: ${query}) {
                edges {
                  node {
                    id
                  }
                }
              }
            }`);
        const orderQueue = res1.orders.edges.map(({ node }) => ({
            orderId: node.id,
            orderParams,
        }));
        const res2 = await throttle(getOrder, orderQueue, 4, 1500);
        const successfulRes = getThrottleResValues(res2);
        return successfulRes;
    }
    catch (e) {
        throw e;
    }
}
exports.getOrders = getOrders;
async function getOrder({ orderId, orderParams, onlyTags = false, }) {
    const res = await exports.graphqlClient.request(`{
            node(id: "${orderId}") {
              id
              ...on Order {
                ${onlyTags ? "tags" : orderParams}
              }
            }
          }`);
    return res.node;
}
exports.getOrder = getOrder;
function formatTax(orderOrOrders) {
    const format = (node) => {
        node.lineItems.edges.forEach(({ node: item }) => {
            const data = {
                priceSet: 0,
                ratePercentage: 0,
            };
            item.taxLines.forEach(({ priceSet: { presentmentMoney: { amount: priceSet }, }, ratePercentage, }) => {
                data.priceSet += parseFloat(priceSet);
                data.ratePercentage += ratePercentage;
            });
            item.taxLine = data;
        });
        return node;
    };
    if (Array.isArray(orderOrOrders)) {
        orderOrOrders.forEach((order) => {
            format(order);
        });
    }
    else {
        format(orderOrOrders);
    }
}
exports.formatTax = formatTax;
async function markOrderAsPaid(orderId) {
    try {
        const res = await exports.graphqlClient.request(`mutation orderMarkAsPaid($input: OrderMarkAsPaidInput!) {
              orderMarkAsPaid(input: $input) {
                order {
                  id
                }
                userErrors {
                  field
                  message
                }
              }
            }`, { input: { id: orderId } });
        return res.orderMarkAsPaid.order;
    }
    catch (e) {
        throw e;
    }
}
exports.markOrderAsPaid = markOrderAsPaid;
async function updateTags(id, ...newTags) {
    const order = await getOrder({
        orderId: id,
        orderParams: `tags`,
        onlyTags: true,
    });
    const existingTags = order.tags;
    const finalTags = [...new Set([...existingTags, ...newTags])];
    const res = await exports.graphqlClient.request(`mutation orderUpdate($input: OrderInput!) {
              orderUpdate(input: $input) {
                userErrors {
                  field
                  message
                }
                order {
                  name
                }
              }
            }`, { input: { id, tags: finalTags } });
    if (res.orderUpdate.order)
        return true;
    return false;
}
exports.updateTags = updateTags;
async function updateTagsForMultipleOrders(orders, ...newTags) {
    const req = async (order) => {
        return await updateTags(order.orderId, ...newTags);
    };
    const res = await throttle(req, orders, 1, 0);
    return getThrottleResValues(res);
}
exports.updateTagsForMultipleOrders = updateTagsForMultipleOrders;
function optimizeOrderCustomAttributes(customAttributes) {
    const map = new Map();
    customAttributes.forEach(({ key, value }) => {
        map.set(key, value);
    });
    return map;
}
exports.optimizeOrderCustomAttributes = optimizeOrderCustomAttributes;
async function throttle(req, reqInputList, limit = 4, wait = 2000) {
    const reqInput = [...reqInputList];
    const results = [];
    while (reqInput.length) {
        const tempInput = [];
        while (tempInput.length < limit + 1 && reqInput.length) {
            const val = reqInput.pop();
            if (val)
                tempInput.push(val);
        }
        const res2 = await Promise.allSettled(tempInput.map((input) => req(input)));
        results.push(...res2);
        if (wait === 0)
            continue;
        await new Promise((resolve) => {
            setTimeout(() => resolve(), wait);
        });
    }
    return results;
}
exports.throttle = throttle;
/** - date must be an ISO string */
function convertISODateToEDIDate(date) {
    return date.split("T")[0].split("-").join("");
}
exports.convertISODateToEDIDate = convertISODateToEDIDate;
function getThrottleResValues(res) {
    return getFulfilledResults(res).map(({ value }) => value);
}
exports.getThrottleResValues = getThrottleResValues;
function getFulfilledResults(res) {
    return res.filter(({ status }) => status === "fulfilled");
}
exports.getFulfilledResults = getFulfilledResults;
